#!/bin/sh -f

## Set DISPLAY Parameter
# AUTOMKDISP=0; Display all images
#            1; Display minimum images
#            2; Display No images
if [ `env | grep AUTOMKDISP |gawk 'END{print(NR)}'` -eq 0 ]; then
AUTOMKDISP=0
fi

## Set Debug Parameter 
# AUTOMKDEBUG=1; erase all temporary files 
#             0; leave all temporary files 
if [ `env | grep AUTOMKDEBUG |gawk 'END{print(NR)}'` -eq 0 ]; then
 AUTOMKDEBUG=1
fi
 
## Parameters for Photometry
# Radius within which detect object to do photometry
PHOTRADIUS=200
# Radius of annulus to measure sky
PHOTAN=10
# Width of annulus to measure sky
PHOTDAN=10

if [ $# -ne 4 ]; then
    echo "Usage : "$0" (FITS data table) (# of dithering point) (Header Name) (ID)"
    exit 1
fi

FTABLE=$1
NPOINT=$2
HEADER=`gawk -v hd=$3 'BEGIN{gsub(".fits","",hd);gsub(".imh","",hd);print(hd)}'`
OUTIM=$HEADER".fits"
ID=$4

PHOTTABLE=RESULTS/phot.$HEADER
OFFSETTABLE=RESULTS/offset.$HEADER

MLCOMBTABLE=mlcomb.tbl

NFRAME=`gawk -v np=$NPOINT 'END{print(NR/np)}' $FTABLE`
echo NFRAME=$NFRAME

MASKTABLE=RESULTS/mask.$HEADER

DITH=20
DITH2=`expr $DITH + $DITH`

TMPFILELIST=tmpfile.list
if [ -f $TMPFILELIST ]; then
  \rm $TMPFILELIST
fi

CLLOG=log/psfmon.$HEADER.`date +%y%m%d%H%M%S`.log
TMPLOG=tmp.log
CLCOM=cl.com

echo $CLCOM >> $TMPFILELIST

if [ ! -d log ]; then
  mkdir log
fi
if [ -f $CLLOG ]; then
  rm $CLLOG
fi

echo "IRAF Log of $0 $1 $2 $3" > $CLLOG

echo "IRAF log will be written in $CLLOG"

# Check Architecture
ARCH=`uname`
uname -a >> $CLLOG

# Check IRAF Version
echo "logout" | cl > $TMPLOG
IRAFVER=`grep Revision $TMPLOG | sed 's/-/ /g' |  gawk '{split($0,a,"Revision");print(a[2])}'| gawk '{split($1,a,".");print(a[1]+a[2]/100+a[3]/1000)}'`
echo "# IRAF Version $IRAFVER "
echo "# IRAF Version $IRAFVER " >> $CLLOG

#Start time
if [ $ARCH = "Linux" ]; then
    SECSTART=`date +%s` 
else 
    SECSTART=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

echo "### Preparation ###"
echo "### Preparation ###" >> $CLLOG

# Check whether image viewer is running.
cat <<EOF > $CLCOM
 display flat.fits 1
 logout
EOF

cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
if [ $ERRSTAT -ne 0 ]; then
 echo "# Image Viewer doesn't seem to be running. #"
 echo "# Change AUTOMKDISP to 2 #" 
 AUTOMKDISP=2
fi

#datapars
cat << EOF > tmpdata.par
scale,r,h,1.,0.
fwhmpsf,r,h,10,0.
emission,b,h,yes
sigma,r,h,INDEF
datamin,r,h,INDEF
datamax,r,h,INDEF
noise,s,h,"poisson"
ccdread,s,h,""
gain,s,h,"GAIN"
readnoise,r,h,15
epadu,r,h,1
exposure,s,h,"EXPTIME"
airmass,s,h,"AIRMASS"
filter,s,h,"FILTER01"
obstime,s,h,""
itime,r,h,1.
xairmass,r,h,INDEF
ifilter,s,h,INDEF
otime,s,h,INDEF
EOF

#centerpars
cat <<EOF > tmpcenter.par
calgorithm,s,h,"centroid"
cbox,r,h,5
cthreshold,r,h,0
minsnratio,r,h,1.,0
cmaxiter,i,h,10
maxshift,r,h,1
clean,b,h,no
rclean,r,h,1
rclip,r,h,2.
kclean,r,h,3
mkcenter,b,h,no
EOF

#fitskypars
cat <<EOF > tmpfitsky.par
salgorithm,s,h,"mode"
annulus,r,h,RADAN
dannulus,r,h,$PHOTDAN
skyvalue,r,h,0.
smaxiter,i,h,10
sloclip,r,h,0.
shiclip,r,h,0.
snreject,i,h,50
sloreject,r,h,3.
shireject,r,h,3.
khist,r,h,3.
binsize,r,h,0.1
smooth,b,h,no
rgrow,r,h,0.
mksky,b,h,no
EOF

#photpars
cat << EOF > tmpphot.par
weighting,s,h,"constant"
apertures,s,h,RADAP
zmag,r,h,25.
mkapert,b,h,no
EOF

cat <<EOF >> $TMPFILELIST
tmpphot.par
tmpdata.par
tmpfitsky.par
tmpcenter.par
tmpphot2.par
tmpfitsky2.par
EOF

TMPPHOT=tmp.phot
TMPPHOT2=tmp2.phot
TMPPSF=tmp.psf
TMPPSF2=tmp2.psf
TMPCUR=tmp.cur
COORDBASE=base.coo
echo $TMPPHOT >> $TMPFILELIST
echo $TMPPHOT2 >> $TMPFILELIST
echo $TMPPSF >> $TMPFILELIST
echo $TMPPSF2 >> $TMPFILELIST
echo $TMPCUR >> $TMPFILELIST
echo $COORDBASE >> $TMPFILELIST

echo "### Photometry ###"
echo "### Photometry ###" >> $CLLOG

gawk '{x[NR]=$1;y[NR]=$2;if(NR==1){xmin=$1;ymin=$2;}else{ if(xmin>$1) xmin=$1;if(ymin>$2) ymin=$2;}}END{for(i=1;i<=NR;i++) printf("%d %d\n",x[i]-xmin,y[i]-ymin)}'  $OFFSETTABLE > tmp.offset
echo tmp.offset >> $TMPFILELIST
XCENTER=`gawk '{if(NR==1){xmax=$1;ymax=$2;}else{if(xmax<$1) xmax=$1;if(ymax<$2) ymax=$2;}}END{print(xmax/2+512)}' tmp.offset`
YCENTER=`gawk '{if(NR==1){xmax=$1;ymax=$2;}else{if(xmax<$1) xmax=$1;if(ymax<$2) ymax=$2;}}END{print(ymax/2+512)}' tmp.offset`
echo "# XCENTER=$XCENTER YCENTER=$YCENTER"

### modify koko
gawk -v id=$ID '{if($1==id)print($2,$3,"-1 1 -1 -1",$1)} ' $MASKTABLE > $COORDBASE

NLINE=`gawk 'END{print(NR)}' $COORDBASE `

ERRSTAT=0

if [ $AUTOMKDISP -lt 2 ]; then
    cat << EOF > $CLCOM
	display RESULTS/${HEADER}.fits 1 fi+
	tvmark 1 $COORDBASE number+
	logout
EOF
    cat $CLCOM | cl >& $TMPLOG
    cat $TMPLOG >> $CLLOG
    ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

    if [ $ERRSTAT != 0 ]; then
      echo "# Error in IRAF ... Exiting #"
      echo "# Check $CLLOG. #" 
      exit 1
    fi
fi


echo "# Measure following stars :"
cat $COORDBASE

echo "# Aperture Photometry in Process ... #"
echo "# Aperture Photometry in Process ... #" >> $CLLOG

if [ -f $PHOTTABLE ]; then
 \rm $PHOTTABLE
fi
echo "#id itime filter epadu, x y fwhm peak, stddev nsky aperture area flux, stddev nsky aperture area flux ...." >> $PHOTTABLE
i=1
while [ $i -le  $NPOINT ]; do
  gawk -v p=$i -v f=$NFRAME '{if(NR>(p-1)*f && NR<=p*f) print $0}' mldata.tbl > mldata.tmp.tbl
  echo mldata.tmp.tbl >> $TMPFILELIST
  XOFFSET=`sed -n ${i}p tmp.offset | gawk '{print($1)}'` 
  YOFFSET=`sed -n ${i}p tmp.offset | gawk '{print($2)}'` 
  COORDFILE=$HEADER.`gawk -v i=$i 'BEGIN{printf("%03d\n",i)}'`.tbl
  gawk -v xo=$XOFFSET -v yo=$YOFFSET '{print($1-xo,$2-yo,$3,$4,$5,$6,$7)}' $COORDBASE > $COORDFILE
  echo $COORDFILE >> $TMPFILELIST
  for filename in `cat mldata.tmp.tbl`; do
    echo " # Measureing $filename #"
    echo " # Measureing $filename #" >> $CLLOG
    j=1
    NAPERTURE=4
    while [ $j -le $NAPERTURE ]; do
      APERTURE=`echo "( $j + 1 ) * 5" | bc`
      ANNULUS=`echo $APERTURE + $PHOTAN | bc`

      sed "s/RADAP/$APERTURE/g" tmpphot.par > tmpphot2.par
      sed "s/RADAN/$ANNULUS/g" tmpfitsky.par > tmpfitsky2.par

      if [ -f $TMPPHOT.$j ]; then
        \rm $TMPPHOT.$j
      fi
      echo $TMPPHOT.$j >> $TMPFILELIST
      cat <<EOF > $CLCOM 
      noao
      digiphot
      apphot
EOF
      if [ $j -eq 1 ]; then
	gawk 'BEGIN{printf("# Coords:")}{printf("\t%s\t%s\n",$1,$2)}' $COORDFILE 
        if [ $AUTOMKDISP -lt 2 ]; then
          cat <<EOF >> $CLCOM 
          display $filename 1
	  tvmark 1 $COORDFILE number+ radii=15 mark=circle
EOF
        fi
      fi
      cat <<EOF >> $CLCOM 
      phot $filename coords=$COORDFILE output=$TMPPHOT.$j interac- verify- fitskyp=tmpfitsky2.par photpars=tmpphot2.par datapars=tmpdata.par centerpars=tmpcenter.par
      logout
EOF
      cat $CLCOM | cl >& $TMPLOG
      cat $TMPLOG >> $CLLOG
      ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`
    
      if [ $ERRSTAT -ne 0 ]; then
	echo "# Error in IRAF ... Exiting #"
	echo "# Check $CLLOG. #" 
	exit 1
      fi

      sed '/^#/d' $TMPPHOT.$j | gawk 'BEGIN{i=1}{j=NR%5;if(j==3)printf("%s %s ",$2,$4);if(j==0){print($1,$3,$2);i++}}' > $TMPPHOT2.$j
      echo $TMPPHOT2.$j >> $TMPFILELIST
      j=`expr $j + 1`
    done

    sed '/^#/d' $TMPPHOT.1 | gawk -v conv=3.5 'BEGIN{i=1}{j=NR%5;if(j==1)printf("%d ",i);if(j==4){printf("%s %s %.2f\n",$1,$3,conv);i++}}' > $TMPPHOT2.com
    sed '/^#/d' $TMPPHOT.1 | gawk '{j=NR%5;if(j==2){printf(": %s %s a\n: center yes\n: fittype moffat\n: iterations 3\n",$1,$2)}}' > $TMPCUR

    echo $TMPPHOT2.com >> $TMPFILELIST

    if [ -f $TMPPSF ]; then
      \rm $TMPPSF
    fi

    cat <<EOF > $CLCOM
      imexam $filename logfile=$TMPPSF keeplog+ graphcur- wcs="physical" graphic="stdgraph" imagecur=$TMPCUR use_dis-
      logout
EOF

    cat $CLCOM | cl >& $TMPLOG
    cat $TMPLOG >> $CLLOG
    ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

    if [ $ERRSTAT -ne 0 ]; then
	echo "# Warning... Failed to fit PSF in $filename #"
	exit 1
    fi

    sed '/^#/d' $TMPPSF | gawk 'BEGIN{i=1}{printf("%s %s %s %s\n",$1,$2,$14,$9);i++}' > $TMPPSF2

    j=1
    NAPERTURE=4
    FILELIST=`echo $TMPPHOT2.com" "$TMPPSF2`
    while [ $j -le $NAPERTURE ]; do
      FILELIST=`echo $FILELIST" "$TMPPHOT2.$j`
      j=`expr $j + 1`
    done
    echo "# $filename " >> $PHOTTABLE
    paste $FILELIST >> $PHOTTABLE
  done
  i=`expr $i + 1`
done


COORDFILE=$COORDBASE
filename=RESULTS/$OUTIM
echo " # Measureing $filename #"
echo " # Measureing $filename #" >> $CLLOG
j=1
NAPERTURE=4
while [ $j -le $NAPERTURE ];do
  APERTURE=`echo "( $j + 1 ) * 5" | bc`
  ANNULUS=`echo $APERTURE + $PHOTAN | bc`

  sed "s/RADAP/$APERTURE/g" tmpphot.par > tmpphot2.par
  sed "s/RADAN/$ANNULUS/g" tmpfitsky.par > tmpfitsky2.par
  if [ -f $TMPPHOT.$j ]; then
    \rm $TMPPHOT.$j
  fi
  cat << EOF > $CLCOM 
    noao
    digiphot
    apphot
    phot $filename coords=$COORDFILE output=$TMPPHOT.$j interac- verify- fitskyp=tmpfitsky2.par photpars=tmpphot2.par datapars=tmpdata.par centerpars=tmpcenter.par
    logout
EOF

  cat $CLCOM | cl >& $TMPLOG
  cat $TMPLOG >> $CLLOG
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $CLLOG. #" 
   exit 1
  fi

  sed '/^#/d' $TMPPHOT.$j | gawk 'BEGIN{i=1}{j=NR%5;if(j==3)printf("%s %s ",$2,$4);if(j==0){print($1,$3,$2);i++}}' > $TMPPHOT2.$j
  j=`expr $j + 1`
done

sed '/^#/d' $TMPPHOT.1 | gawk -v conv=3.5 'BEGIN{i=1}{j=NR%5;if(j==1)printf("%d ",i);if(j==4){printf("%s %s %.2f\n",$1,$3,conv);i++}}' > $TMPPHOT2.com
sed '/^#/d' $TMPPHOT.1 | gawk '{j=NR%5;if(j==2){printf(": %s %s a\n: center yes\n: fittype moffat\n: iterations 3\n",$1,$2)}}' > $TMPCUR
echo 
if [ -f $TMPPSF ]; then
  \rm $TMPPSF
fi
cat <<EOF > $CLCOM 
 imexam $filename logfile=$TMPPSF keeplog+ graphcur- wcs="physical" graphic="stdgraph" imagecur=$TMPCUR use_dis-
 logout
EOF

cat $CLCOM | cl >& $TMPLOG
cat $TMPLOG >> $CLLOG
ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

if [ $ERRSTAT != 0 ]; then
    echo "# Warning... Failed to fit PSF in $filename #"
#    exit 1
fi

sed '/^#/d' $TMPPSF | gawk 'BEGIN{i=1}{printf("%s %s %s %s\n",$1,$2,$14,$9);i++}' > $TMPPSF2

NAPERTURE=4
FILELIST=`echo $TMPPHOT2.com" "$TMPPSF2`

#--
j=1
while [ $j -le $NAPERTURE ]; do
  FILELIST=`echo $FILELIST" "$TMPPHOT2.$j`
  j=`expr $j + 1`
done

echo "# $filename " >> $PHOTTABLE
paste $FILELIST >> $PHOTTABLE

j=1
while [ $j -le  $NAPERTURE ]; do
  echo $TMPPHOT.$j >> $TMPFILELIST
  echo $TMPPHOT2.$j >> $TMPFILELIST
  j=`expr $j + 1`
done

echo "### $0 Finished ###"
echo "### $0 Finished ###" >> $CLLOG
echo "IRAF log written in $CLLOG"
echo "Results of photometry are saved in $PHOTTABLE"
if [ $AUTOMKDEBUG -eq 1 ]; then
  for filename in `cat $TMPFILELIST`; do
   if [ -f $filename ]; then
      \rm $filename
   fi
  done
fi

#END time
if [ $ARCH = "Linux" ]; then
    SECEND=`date +%s` 
else
    SECEND=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

echo $SECSTART $SECEND | gawk '{printf("Took %d sec to complete\n",$2-$1)}' >> $CLLOG
echo $SECSTART $SECEND | gawk '{printf("Took %d sec to complete\n",$2-$1)}'
